=== modified file 'breezy/git/workingtree.py'
--- breezy/git/workingtree.py	2018-09-01 19:51:29 +0000
+++ breezy/git/workingtree.py	2018-09-21 01:05:50 +0000
@@ -729,7 +729,7 @@
 
     def is_executable(self, path, file_id=None):
         with self.lock_read():
-            if getattr(self, "_supports_executable", osutils.supports_executable)():
+            if self._supports_executable():
                 mode = self._lstat(path).st_mode
             else:
                 (index, subpath) = self._lookup_index(path.encode('utf-8'))
@@ -740,7 +740,7 @@
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
     def _is_executable_from_path_and_stat(self, path, stat_result):
-        if getattr(self, "_supports_executable", osutils.supports_executable)():
+        if self._supports_executable():
             return self._is_executable_from_path_and_stat_from_stat(path, stat_result)
         else:
             return self._is_executable_from_path_and_stat_from_basis(path, stat_result)

=== modified file 'breezy/osutils.py'
--- breezy/osutils.py	2018-09-11 07:31:44 +0000
+++ breezy/osutils.py	2018-09-21 01:05:50 +0000
@@ -31,6 +31,7 @@
 import locale
 import ntpath
 import posixpath
+import psutil
 import select
 # We need to import both shutil and rmtree as we export the later on posix
 # and need the former on windows
@@ -1649,8 +1650,14 @@
     _terminal_size = _ioctl_terminal_size
 
 
-def supports_executable():
-    return sys.platform != "win32"
+def supports_executable(path):
+    if sys.platform == 'win32':
+        return False
+    import pdb; pdb.set_trace()
+    if get_fs_type(path) in ('vfat', 'ntfs'):
+        # filesystems known to not support executable bit
+        return False
+    return True
 
 
 def supports_posix_readonly():
@@ -2576,3 +2583,11 @@
     if sys.platform == "win32" and win32utils._is_pywintypes_error(evalue):
         return True
     return False
+
+
+def get_fs_type(path):
+    """Return the filesystem type for the partition a path is in."""
+    import psutil
+    for part in sorted(psutil.disk_partitions(), key=lambda x: len(x.mountpoint), reverse=True):
+        if is_inside(part.mountpoint, path):
+            return part.fstype

=== modified file 'breezy/transform.py'
--- breezy/transform.py	2018-08-18 16:29:34 +0000
+++ breezy/transform.py	2018-09-21 01:05:50 +0000
@@ -1257,8 +1257,7 @@
 
     def _limbo_supports_executable(self):
         """Check if the limbo path supports the executable bit."""
-        # FIXME: Check actual file system capabilities of limbodir
-        return osutils.supports_executable()
+        return osutils.supports_executable(self._limbodir)
 
     def _limbo_name(self, trans_id):
         """Generate the limbo name of a file"""

=== modified file 'breezy/workingtree.py'
--- breezy/workingtree.py	2018-09-14 14:13:30 +0000
+++ breezy/workingtree.py	2018-09-21 01:05:50 +0000
@@ -157,10 +157,7 @@
         return self._format.supports_merge_modified
 
     def _supports_executable(self):
-        if sys.platform == 'win32':
-            return False
-        # FIXME: Ideally this should check the file system
-        return True
+        return osutils.supports_executable(self.basedir)
 
     def break_lock(self):
         """Break a lock if one is present from another instance.

